package yrpg.fi.cas.plugin.other.task;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.exception.KDException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import yrpg.fi.cas.plugin.other.webservice.OutServiceLocator;
import yrpg.fi.cas.plugin.other.webservice.OutServicePortType;
import java.util.Map;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 后台任务插件
 */
public class PushCaspayBillToBaoRongTask extends AbstractTask implements Plugin {

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {

        //盐值  salt
        String salt=System.getProperty("yrpg.fi.cas.baorongzj.salt");

        String url=System.getProperty("yrpg.fi.cas.baorongzj.url");

        //对接系统代码   srcoutsystemcode
        String srcoutsystemcode=System.getProperty("yrpg.fi.cas.baorongzj.srcoutsystemcode");

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        Date now=new Date();

        //付款处理单据状态为“已审核”、支付状态为“1-未推送或6-推送失败或者8-重付待推”且是送否推送保融支付为是的单据
        QFilter statusFilter = new QFilter("billstatus", QCP.equals, "C");//单据状态
        //支付状态
        List<String> paystatus = new ArrayList<>();
        paystatus.add("1");//未推送
        paystatus.add("6");//推送失败
        paystatus.add("8");//重付待推
        QFilter paystatusFilter = new QFilter("yrpg_paystatus", QCP.in, paystatus);//支付状态
        QFilter pushfundpayFilter = new QFilter("yrpg_pushfundpay", QCP.equals, true);//是否推送保融支付

        QFilter[] caspaybillfilters = new QFilter[]{statusFilter, paystatusFilter, pushfundpayFilter};
        DynamicObject[] caspaybills = BusinessDataServiceHelper.load("cas_paybill", "id,yrpg_brcode,payeebank,billno,billstatus,yrpg_paystatus,yrpg_pushfailurecaus,openorg.number,applyorg.number,expectdate,yrpg_transactiontype,settletype,settletype.number,settletype.yrpg_settlementtypcode,yrpg_brbillingmethodcode,urgentflag,payeracctbank.bankaccountnumber,actpayamt,payeename,payeebanknum,ispersonpay", caspaybillfilters);

        if (caspaybills.length > 0) {
            for (DynamicObject caspaybill : caspaybills) {

                long caspayid = caspaybill.getLong("id");//付款处理单id
                String billno = caspaybill.getString("billno");//付款处理单编码
                DynamicObject openorg=caspaybill.getDynamicObject("openorg");//核算组织
                String openorgno ="";//核算组织编码
                if(openorg!=null){
                    openorgno=openorg.getString("number");
                }
                DynamicObject applyorg=caspaybill.getDynamicObject("applyorg");//申请组织
                String applyorgno ="";
                if(applyorg!=null){
                    applyorgno=applyorg.getString("number");
                }
                Date expectdate = caspaybill.getDate("expectdate");//期望付款日期
                DynamicObject transactiontype=caspaybill.getDynamicObject("yrpg_transactiontype");//交易类型
                String transactiontypeno = "";//交易类型编码
                if(transactiontype!=null){
                    transactiontypeno=transactiontype.getString("number");
                }

                //收款银行编码
                String payeebankno="";

                //收款银行
                DynamicObject payeebank=caspaybill.getDynamicObject("payeebank");
                if(payeebank!=null){
                    payeebankno=payeebank.getString("number");
                }

                //版本号
                int brcode=caspaybill.getInt("yrpg_brcode");

                //结算方式
                String brbillingmethodcode="";//结算方式代码
                DynamicObject settletype=caspaybill.getDynamicObject("settletype");
                if(settletype!=null){
                    brbillingmethodcode=settletype.getString("yrpg_settlementtypcode");
                }

                Boolean urgentflag = caspaybill.getBoolean("urgentflag");//加急支付
                int urgentfl=0;
                if(urgentflag){
                    urgentfl=1;
                }

                DynamicObject payeracctbank=caspaybill.getDynamicObject("payeracctbank");//银行账号
                String payeracctbankno = "";
                if(payeracctbank!=null){
                    payeracctbankno=payeracctbank.getString("bankaccountnumber");
                }

                String currencyno ="CNY";//币别
                BigDecimal actpayamt = caspaybill.getBigDecimal("actpayamt");//付款金额
                String recaccbankname = caspaybill.getString("payeename");//收款人实名
                String payeebanknum = caspaybill.getString("payeebanknum");//收款人账号
                Boolean ispersonpay = caspaybill.getBoolean("ispersonpay");//对私支付 true：对私  false:对公
                int ispersonp=2;
                if(ispersonpay){
                    ispersonp=1;
                }

                String detail ="<detail>"+
                        "<SRCSERIALNO>"+caspayid+"</SRCSERIALNO>"+
                        "<SRCNOTECODE>"+billno+"</SRCNOTECODE>"+
                        "<ORGCODE>"+openorgno+"</ORGCODE>"+
                        "<APPLYORGCODE>"+applyorgno+"</APPLYORGCODE>"+
                        "<PAYDATE>"+sdf.format(expectdate)+"</PAYDATE>"+
                        "<PAYTYPECODE>"+transactiontypeno+"</PAYTYPECODE>"+
                        "<SETTLEMENTMODECODE>"+brbillingmethodcode+"</SETTLEMENTMODECODE>"+
                        "<OPPBANKLOCATIONCODE>"+payeebankno+"</OPPBANKLOCATIONCODE>"+
                        "<ISURGENT>"+urgentfl+"</ISURGENT>"+
                        "<OURORGCODE>"+applyorgno+"</OURORGCODE>"+
                        "<OURBANKACCOUNTNUMBER>"+payeracctbankno+"</OURBANKACCOUNTNUMBER>"+
                        "<OURCURCODE>"+currencyno+"</OURCURCODE>"+
                        "<OURAMOUNT>"+actpayamt+"</OURAMOUNT>"+
                        "<OPPOBJECTNAME>"+recaccbankname+"</OPPOBJECTNAME>"+
                        "<OPPBANKACCOUNTNUMBER>"+payeebanknum+"</OPPBANKACCOUNTNUMBER>"+
                        "<OPPBANKACCOUNTNAME>"+recaccbankname+"</OPPBANKACCOUNTNAME>"+
                        "<OPPDIRECTCURCODE>"+currencyno+"</OPPDIRECTCURCODE>"+
                        "<OPPPRIVATEFLAG>"+ispersonp+"</OPPPRIVATEFLAG>"+
                        "</detail>";

                //req参数
                String req="<req><head><ALLAMOUNT>"+actpayamt+"</ALLAMOUNT><ALLCOUNT>"+1+"</ALLCOUNT></head>"+
                        "<list>"+detail.toString()+"</list></req>";

                //MD5值
                String md5str=md5String(req,salt);

                //pub参数
                String pub="<pub>" +
                        "<SRCOUTSYSTEMCODE>"+srcoutsystemcode+"</SRCOUTSYSTEMCODE>" +
                        "<SRCBATCHNO>"+caspayid+"-"+brcode+"</SRCBATCHNO> " +
                        "<TRANSCODE>PMSQ01</TRANSCODE>" +
                        "<TRANSDATETIME>"+sdf.format(now)+"</TRANSDATETIME>" +
                        "<MD5>"+md5str+"</MD5>" +
                        "</pub>";


                //入参
                String xml="<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                        "<MBS>"+pub+req+"</MBS>";

                Map<String, Object> logMap = new HashMap<>();
                logMap.put("opname", "定时任务推送付款处理到资金接口");
                logMap.put("thirdappname", "资金接口");
                logMap.put("opdesc",url);
                logMap.put("request", xml);
                logMap.put("status", "true");
                logMap.put("username", "yy");

                try {
                    OutServiceLocator locatar = new OutServiceLocator();
                    OutServicePortType outServiceHttpPort = locatar.getOutServiceHttpPort();
                    String response =outServiceHttpPort.outSystemWS(xml);

                    Document document = DocumentHelper.parseText(response);
                    Element rootElement = document.getRootElement();
                    Element pubele = rootElement.element("pub");

                    //返回代码
                    Element rescodeele=pubele.element("RESPCODE");
                    String rescode=(String)rescodeele.getData();

                    //失败信息
                    Element respmsgele=pubele.element("RESPMSG");
                    String respmsg=(String)respmsgele.getData();

                    if("0".equals(rescode)){//"0"成功
                        caspaybill.set("billstatus","E");//单据状态 付款处理中
                        caspaybill.set("yrpg_paystatus","4");//支付状态 支付中
                        caspaybill.set("yrpg_brcode",brcode);
                        caspaybill.set("yrpg_pushfailurecaus",respmsg);//失败原因

                        SaveServiceHelper.update(caspaybill);//更新付款处理单

                        logMap.put("response", "推送付款处理单成功！");
                        saveApiLog(logMap);
                    }else{//失败
                        caspaybill.set("billstatus","C");//单据状态 已审核
                        caspaybill.set("yrpg_paystatus","6");//支付状态  推送失败
                        caspaybill.set("yrpg_pushfailurecaus",respmsg);//失败原因
                        brcode++;
                        caspaybill.set("yrpg_brcode",brcode);

                        SaveServiceHelper.update(caspaybill);//更新付款处理单

                        logMap.put("status", "false");
                        logMap.put("response", "推送付款处理失败！错误信息是"+respmsg);

                        saveApiLog(logMap);
                    }

                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }

        }
    }

    /**
     * 对字符串进行md5加密
     * @param md5String
     * @param md5Key
     * @return md5加密后的密文
     */
    private static String md5String(String md5String,String md5Key){
        String md5Value = MD5Encoder(md5String + md5Key);
        return md5Value;
    }

    public final static String MD5Encoder(String s) {
        return MD5Encoder(s,"utf-8");
    }

    public final static String MD5Encoder(String s, String charset) {
        try {
            byte[] btInput = s.getBytes(charset);
            MessageDigest mdInst = MessageDigest.getInstance("MD5");
            mdInst.update(btInput);
            byte[] md = mdInst.digest();
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < md.length; i++) {
                int val = ((int) md[i]) & 0xff;
                if (val < 16){
                    sb.append("0");
                }
                sb.append(Integer.toHexString(val));
            }
            return sb.toString();
        } catch (Exception e) {
            return null;
        }
    }

    public static void saveApiLog(Map<String,Object> logMap){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_data");
        //api编号
        apiLog.set("opname",logMap.get("opname"));
        //URL
        apiLog.set("opdesc",logMap.get("opdesc"));
        //请求参数
        if(logMap.get("request")!=null){
            apiLog.set("request",logMap.get("request").toString().length()>2000?logMap.get("request").toString().substring(0, 2000):logMap.get("request"));
        }else {
            apiLog.set("request","");
        }

        if (logMap.get("response")!=null){
            //响应结果
            apiLog.set("response",logMap.get("response").toString().length()>2000?logMap.get("response").toString().substring(0, 2000):logMap.get("response"));
        }else {
            apiLog.set("response","");
        }
        //调用时间
        apiLog.set("opdate",new Date());
        //调用者
        apiLog.set("username",logMap.get("username"));
        //第三方应用
        apiLog.set("thirdappname",logMap.get("thirdappname"));
        //调用状态
        apiLog.set("status",logMap.get("status"));
        //保存日志
        SaveServiceHelper.save(new DynamicObject[]{apiLog});

    }
}